import { List, PullRefresh } from '@taroify/core';
import classNames from 'classnames';
import { useCallback, useEffect, useRef, useState } from 'react';

import ListPlaceholder from '@/components/list-placeholder';
import { ProfitStatusDescriptions } from '@/constants/partner';
import { GetProfitRequest, PartnerProfitItem, ProfitType } from '@/types/partner';
import { logWithPrefix } from '@/utils/common';
import { formatMoney, formatTimestamp, getProfitList as requestData } from '@/utils/partner';

import './index.less';

export interface IPartnerProfitListProps extends GetProfitRequest {
  visible?: boolean;
  refreshDisabled?: boolean;
  listHeight?: number;
  className?: string;
  onListEmpty?: () => void;
}

const PREFIX = 'partner-profit';
const log = logWithPrefix(PREFIX);

function ProfitList(props: IPartnerProfitListProps) {
  const { className, listHeight, refreshDisabled, visible = true, type, onListEmpty } = props;
  const [refreshing, setRefreshing] = useState(false);
  const [loadingMore, setLoadingMore] = useState(false);
  const [loadMoreError, setLoadMoreError] = useState(false);
  const [dataList, setDataList] = useState<PartnerProfitItem[]>([]);
  const onListEmptyRef = useRef(onListEmpty);

  const handleRefresh = useCallback(async () => {
    log('start pull refresh');
    try {
      setRefreshing(true);
      setLoadMoreError(false);
      const list = await requestData({ type });
      setDataList(list);
      !list.length && onListEmptyRef.current?.();
      log('pull refresh success');
    } catch (e) {
      setDataList([]);
      setLoadMoreError(true);
      log('pull refresh failed');
    } finally {
      setRefreshing(false);
    }
  }, []);

  useEffect(() => {
    onListEmptyRef.current = onListEmpty;
  }, [onListEmpty]);

  // 初始化数据&配置变更后刷新数据
  useEffect(() => {
    // 列表不可见时，先不做处理
    if (!visible) {
      log('visible changed, but is not visible, only clear list');
      return;
    }

    const refresh = async () => {
      log('visible changed, start refresh list data');
      try {
        setDataList([]);
        setLoadingMore(true);
        setLoadMoreError(false);
        const list = await requestData({ type });
        setDataList(list);
        !list.length && onListEmptyRef.current?.();
      } catch (e) {
        setDataList([]);
        setLoadMoreError(true);
      } finally {
        log('visible changed, refresh list data end');
        setLoadingMore(false);
      }
    };
    refresh();
  }, [visible, type]);

  return (
    <div className={`${PREFIX}__tab-content`}>
      <PullRefresh
        className={classNames(`${PREFIX}__pull-refresh`, className)}
        loading={refreshing}
        onRefresh={handleRefresh}
        disabled={refreshDisabled}
      >
        <List
          hasMore={false}
          onLoad={() => {}}
          loading={loadingMore || refreshing}
          disabled={loadMoreError}
          fixedHeight={typeof listHeight !== 'undefined'}
          style={listHeight ? { height: `${listHeight}px` } : undefined}
        >
          {dataList.map(item => {
            const isChat =
              type === ProfitType.CHAT_SHARE || item.earnType.toString().toLowerCase().indexOf('chat') > -1;
            return (
              <div className={`${PREFIX}__row`} key={item.id}>
                <div className={`${PREFIX}__row-content`}>
                  <div className={`${PREFIX}__item time`}>{formatTimestamp(item.created)}</div>
                  <div className={`${PREFIX}__item project`}>{isChat ? '主播被开聊' : '会员支付'}</div>
                  <div className={`${PREFIX}__item status`}>
                    {isChat ? '已结算' : ProfitStatusDescriptions[item.status]}
                  </div>
                  <div className={`${PREFIX}__item income`}>+{formatMoney(item.amount)}</div>
                </div>
              </div>
            );
          })}
          <ListPlaceholder hasMore={false} loadingMore={loadingMore} loadMoreError={loadMoreError} />
        </List>
      </PullRefresh>
    </div>
  );
}

export default ProfitList;
